package org.xbib.interlibrary.elasticsearch.sru;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.elasticsearch.search.aggregations.AggregationBuilder;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.aggregations.BucketOrder;
import org.xbib.cql.elasticsearch.FacetsGenerator;
import org.xbib.cql.elasticsearch.Visitor;
import org.xbib.cql.elasticsearch.ast.Expression;
import org.xbib.cql.elasticsearch.ast.Modifier;
import org.xbib.cql.elasticsearch.ast.Name;
import org.xbib.cql.elasticsearch.ast.Operator;
import org.xbib.cql.elasticsearch.ast.Token;

public class ElasticsearchFacetsGenerator extends FacetsGenerator implements Visitor {

    private final List<AggregationBuilder> list;

    public ElasticsearchFacetsGenerator() throws IOException {
        super();
        this.list = new ArrayList<>();
    }

    public List<AggregationBuilder> getList() {
        return list;
    }

    @Override
    public void visit(Token node) {
    }

    @Override
    public void visit(Name node) {
    }

    @Override
    public void visit(Modifier node) {
    }

    @Override
    public void visit(Operator node) {
    }

    @Override
    public void visit(Expression node) {
    }

    @Override
    public void facet(String facetLimit, String facetSort) {
        if (facetLimit == null) {
            return;
        }
        Map<String, Integer> facetMap = parseFacet(facetLimit, 10);
        String[] sortSpec = facetSort != null ? facetSort.split(",") : new String[]{"recordCount", "descending"};
        String order = "count";
        boolean asc = false;
        for (String s : sortSpec) {
            switch (s) {
                case "recordCount":
                    order = "count";
                    break;
                case "alphanumeric":
                    order = "key";
                    break;
                case "ascending":
                    asc = true;
                    break;
                case "descending":
                    asc = false;
                    break;
            }
        }
        BucketOrder bucketOrder = BucketOrder.count(asc);
        if ("key".equals(order)) {
            bucketOrder = BucketOrder.key(asc);
        }
        for (String index : facetMap.keySet()) {
            if ("*".equals(index)) {
                continue;
            }
            // TODO(jprante) range aggregations etc.
            Integer size = facetMap.get(index);
            AggregationBuilder builder = AggregationBuilders.terms(index)
                    .field(index)
                    .size(size > 0 ? size : 10)
                    .order(bucketOrder);
            list.add(builder);
        }
    }

    private Map<String, Integer> parseFacet(String spec, int defaultFacetLimit) {
        Map<String, Integer> m = new HashMap<>();
        m.put("*", defaultFacetLimit);
        if (spec == null || spec.length() == 0) {
            return m;
        }
        String[] params = spec.split(",");
        for (String param : params) {
            int pos = param.indexOf(':');
            if (pos > 0) {
                int n = parseInt(param.substring(0, pos), defaultFacetLimit);
                m.put(param.substring(pos + 1), n);
            } else if (param.length() > 0) {
                int n = parseInt(param, defaultFacetLimit);
                m.put("*", n);
            }
        }
        return m;
    }

    private int parseInt(String s, int defaultValue) {
        try {
            return Integer.parseInt(s);
        } catch (NumberFormatException e) {
            return defaultValue;
        }
    }
}
