package org.xbib.interlibrary.elasticsearch.sru;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;
import org.elasticsearch.search.sort.FieldSortBuilder;
import org.elasticsearch.search.sort.SortBuilder;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.xbib.cql.elasticsearch.SortGenerator;
import org.xbib.cql.elasticsearch.Visitor;
import org.xbib.cql.elasticsearch.ast.Expression;
import org.xbib.cql.elasticsearch.ast.Modifier;
import org.xbib.cql.elasticsearch.ast.Name;
import org.xbib.cql.elasticsearch.ast.Node;
import org.xbib.cql.elasticsearch.ast.Operator;
import org.xbib.cql.elasticsearch.ast.Token;

/**
 * Build sort in Elasticsearch JSON syntax from abstract syntax tree.
 */
public class ElasticsearchSortGenerator extends SortGenerator implements Visitor {

    private final Stack<Modifier> modifiers;

    private final List<SortBuilder<?>> list;

    public ElasticsearchSortGenerator() {
        super();
        modifiers = new Stack<>();
        list = new ArrayList<>();
    }

    public List<SortBuilder<?>> getList() {
        return list;
    }

    @Override
    public void visit(Token node) {
    }

    @Override
    public void visit(Name node) {
        if (modifiers.isEmpty()) {
            list.add(SortBuilders.fieldSort(node.getName())
                    .unmappedType("keyword")
                    .missing("_last"));
        } else {
            FieldSortBuilder builder = SortBuilders.fieldSort(node.getName());
            while (!modifiers.isEmpty()) {
                Modifier mod = modifiers.pop();
                String s = mod.getName().toString();
                switch (s) {
                    case "ascending":
                    case "sort.ascending": {
                        builder.order(SortOrder.ASC);
                        break;
                    }
                    case "descending":
                    case "sort.descending": {
                        builder.order(SortOrder.DESC);
                        break;
                    }
                    default: {
                        //builder.field(s, mod.getTerm().toString());
                        break;
                    }
                }
            }
            builder.unmappedType("keyword").missing("_last");
            list.add(builder);
        }
    }

    @Override
    public void visit(Modifier node) {
        modifiers.push(node);
    }

    @Override
    public void visit(Operator node) {
    }

    @Override
    public void visit(Expression node) {
        Operator op = node.getOperator();
        if (op == Operator.SORT) {
            for (Node arg : node.getArgs()) {
                arg.accept(this);
            }
        }
    }

}
