package org.xbib.interlibrary.elasticsearch.sru;

import java.io.IOException;
import java.util.Map;
import java.util.function.Predicate;
import org.xbib.elx.api.SearchClient;
import org.xbib.elx.common.ClientBuilder;
import org.xbib.elx.transport.TransportSearchClientProvider;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.settings.Settings;
import org.xbib.settings.SettingsBuilder;

public class TransportClientHelper {

    public SearchClient createSearchClient(ServiceArguments arguments) throws IOException {
        return createSearchClient(arguments.getSettings().getAsSettings("elasticsearch." + arguments.getName()));
    }

    public SearchClient createSearchClient(Settings settings) throws IOException {
        return ClientBuilder.builder()
                .setSearchClientProvider(TransportSearchClientProvider.class)
                .put(filter(settings, s -> !isPrivateSettings(s)).getAsMap())
                .build();
    }

    private static Settings filter(Settings settings, Predicate<String> predicate) {
        SettingsBuilder builder = Settings.settingsBuilder();
        for (Map.Entry<String, String> entry : settings.getAsMap().entrySet()) {
            if (predicate.test(entry.getKey())) {
                builder.put(entry.getKey(), entry.getValue());
            }
        }
        return builder.build();
    }

    private static boolean isPrivateSettings(String key) {
        for (ServiceParameters p : ServiceParameters.values()) {
            if (key.equals(p.getName())) {
                return true;
            }
        }
        return false;
    }
}
